/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package NZGPS;

import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.location.*;
import java.lang.Math.*;
import java.util.Date;
import java.util.Date.*;

class convertCoords {
    /* Defines PI (from Abramowitz and Stegun Table 1.1) */

    static double PI = 3.1415926535898;
    static double TWOPI = (2.0 * PI);
    static double rad2deg = (180 / PI);
    static double NZTM_A = 6378137;
    static double NZTM_RF = 298.257222101;
    static double NZTM_CM = 173.0;
    static double NZTM_OLAT = 0.0;
    static double NZTM_SF = 0.9996;
    static double NZTM_FE = 1600000.0;
    static double NZTM_FN = 10000000.0;

    /* Structure used to define a TM projection */
    class tmprojection {

        public double meridian;          /* Central meridian */

        public double scalef;            /* Scale factor */

        public double orglat;            /* Origin latitude */

        public double falsee;            /* False easting */

        public double falsen;            /* False northing */

        public double utom;              /* Unit to metre conversion */

        public double a, rf, f, e2, ep2;     /* Ellipsoid parameters */

        public double om;                /* Intermediate calculation */

    }

//static double meridian_arc( tmprojection tm, double lt );

    /* Initiallize the TM structure  */
    static void define_tmprojection(tmprojection tm, double a, double rf,
            double cm, double sf, double lto, double fe, double fn, double utom) {

        double f;

        tm.meridian = cm;
        tm.scalef = sf;
        tm.orglat = lto;
        tm.falsee = fe;
        tm.falsen = fn;
        tm.utom = utom;
        if (rf != 0.0) {
            f = 1.0 / rf;
        } else {
            f = 0.0;
        }
        tm.a = a;
        tm.rf = rf;
        tm.f = f;
        tm.e2 = 2.0 * f - f * f;
        tm.ep2 = tm.e2 / (1.0 - tm.e2);

        tm.om = meridian_arc(tm, tm.orglat);
    }

    /***************************************************************************/
    /*                                                                         */
    /*  meridian_arc                                                           */
    /*                                                                         */
    /*  Returns the length of meridional arc (Helmert formula)                 */
    /*  Method based on Redfearn's formulation as expressed in GDA technical   */
    /*  manual at http://www.anzlic.org.au/icsm/gdatm/index.html               */
    /*                                                                         */
    /*  Parameters are                                                         */
    /*    projection                                                           */
    /*    latitude (radians)                                                   */
    /*                                                                         */
    /*  Return value is the arc length in metres                               */
    /*                                                                         */
    /***************************************************************************/
    static double meridian_arc(tmprojection tm, double lt) {
        double e2 = tm.e2;
        double a = tm.a;
        double e4;
        double e6;
        double A0;
        double A2;
        double A4;
        double A6;

        e4 = e2 * e2;
        e6 = e4 * e2;

        A0 = 1 - (e2 / 4.0) - (3.0 * e4 / 64.0) - (5.0 * e6 / 256.0);
        A2 = (3.0 / 8.0) * (e2 + e4 / 4.0 + 15.0 * e6 / 128.0);
        A4 = (15.0 / 256.0) * (e4 + 3.0 * e6 / 4.0);
        A6 = 35.0 * e6 / 3072.0;

        return a * (A0 * lt - A2 * Math.sin(2 * lt) + A4 * Math.sin(4 * lt) - A6 * Math.sin(6 * lt));
    }

    /*************************************************************************/
    /*                                                                       */
    /*   foot_point_lat                                                      */
    /*                                                                       */
    /*   Calculates the foot point latitude from the meridional arc          */
    /*   Method based on Redfearn's formulation as expressed in GDA technical*/
    /*   manual at http://www.anzlic.org.au/icsm/gdatm/index.html            */
    /*                                                                       */
    /*   Takes parameters                                                    */
    /*      tm definition (for scale factor)                                 */
    /*      meridional arc (metres)                                          */
    /*                                                                       */
    /*   Returns the foot point latitude (radians)                           */ /*                                                                       */

    /*************************************************************************/
    static double foot_point_lat(tmprojection tm, double m) {
        double f = tm.f;
        double a = tm.a;
        double n;
        double n2;
        double n3;
        double n4;
        double g;
        double sig;
        double phio;

        n = f / (2.0 - f);
        n2 = n * n;
        n3 = n2 * n;
        n4 = n2 * n2;

        g = a * (1.0 - n) * (1.0 - n2) * (1 + 9.0 * n2 / 4.0 + 225.0 * n4 / 64.0);
        sig = m / g;

        phio = sig + (3.0 * n / 2.0 - 27.0 * n3 / 32.0) * Math.sin(2.0 * sig)
                + (21.0 * n2 / 16.0 - 55.0 * n4 / 32.0) * Math.sin(4.0 * sig)
                + (151.0 * n3 / 96.0) * Math.sin(6.0 * sig)
                + (1097.0 * n4 / 512.0) * Math.sin(8.0 * sig);

        return phio;
    }

    /***************************************************************************/
    /*                                                                         */
    /*   tmgeod                                                                */
    /*                                                                         */
    /*   Routine to convert from Tranverse Mercator to latitude and longitude. */
    /*   Method based on Redfearn's formulation as expressed in GDA technical  */
    /*   manual at http://www.anzlic.org.au/icsm/gdatm/index.html              */
    /*                                                                         */
    /*   Takes parameters                                                      */
    /*      input easting (metres)                                             */
    /*      input northing (metres)                                            */
    /*      output latitude (radians)                                          */
    /*      output longitude (radians)                                         */
    /*                                                                         */
    /***************************************************************************/

    /*static void tm_geod( tmprojection *tm,
    double ce, double cn, double *ln, double *lt ) {
    double fn = tm.falsen;
    double fe = tm.falsee;
    double sf = tm.scalef;
    double e2 = tm.e2;
    double a = tm.a;
    double cm = tm.meridian;
    double om = tm.om;
    double utom = tm.utom;
    double cn1;
    double fphi;
    double slt;
    double clt;
    double eslt;
    double eta;
    double rho;
    double psi;
    double E;
    double x;
    double x2;
    double t;
    double t2;
    double t4;
    double trm1;
    double trm2;
    double trm3;
    double trm4;
    
    cn1  =  (cn - fn)*utom/sf + om;
    fphi = foot_point_lat(tm, cn1);
    slt = Math.sin(fphi);
    clt = Math.cos(fphi);
    
    eslt = (1.0-e2*slt*slt);
    eta = a/sqrt(eslt);
    rho = eta * (1.0-e2) / eslt;
    psi = eta/rho;
    
    E = (ce-fe)*utom;
    x = E/(eta*sf);
    x2 = x*x;
    
    
    t = slt/clt;
    t2 = t*t;
    t4 = t2*t2;
    
    trm1 = 1.0/2.0;
    
    trm2 = ((-4.0*psi
    +9.0*(1-t2))*psi
    +12.0*t2)/24.0;
    
    trm3 = ((((8.0*(11.0-24.0*t2)*psi
    -12.0*(21.0-71.0*t2))*psi
    +15.0*((15.0*t2-98.0)*t2+15))*psi
    +180.0*((-3.0*t2+5.0)*t2))*psi + 360.0*t4)/720.0;
    
    trm4 = (((1575.0*t2+4095.0)*t2+3633.0)*t2+1385.0)/40320.0;
    
     *lt = fphi+(t*x*E/(sf*rho))*(((trm4*x2-trm3)*x2+trm2)*x2-trm1);
    
    trm1 = 1.0;
    
    trm2 = (psi+2.0*t2)/6.0;
    
    trm3 = (((-4.0*(1.0-6.0*t2)*psi
    +(9.0-68.0*t2))*psi
    +72.0*t2)*psi
    +24.0*t4)/120.0;
    
    trm4 = (((720.0*t2+1320.0)*t2+662.0)*t2+61.0)/5040.0;
    
     *ln = cm - (x/clt)*(((trm4*x2-trm3)*x2+trm2)*x2-trm1);
    }*/
    /***************************************************************************/
    /*                                                                         */
    /*   geodtm                                                                */
    /*                                                                         */
    /*   Routine to convert from latitude and longitude to Transverse Mercator.*/
    /*   Method based on Redfearn's formulation as expressed in GDA technical  */
    /*   manual at http://www.anzlic.org.au/icsm/gdatm/index.html              */
    /*   Loosely based on FORTRAN source code by J.Hannah and A.Broadhurst.    */
    /*                                                                         */
    /*   Takes parameters                                                      */
    /*      input latitude (radians)                                           */
    /*      input longitude (radians)                                          */
    /*      output easting  (metres)                                           */
    /*      output northing (metres)                                           */
    /*                                                                         */
    /***************************************************************************/
    class TMpoint {

        public double ce, cn;
    }

    TMpoint geod_tm(tmprojection tm,
            double ln, double lt /*double *ce, double *cn*/) {
        TMpoint out = new TMpoint();
        double fn = tm.falsen;
        double fe = tm.falsee;
        double sf = tm.scalef;
        double e2 = tm.e2;
        double a = tm.a;
        double cm = tm.meridian;
        double om = tm.om;
        double utom = tm.utom;
        double dlon;
        double m;
        double slt;
        double eslt;
        double eta;
        double rho;
        double psi;
        double clt;
        double w;
        double wc;
        double wc2;
        double t;
        double t2;
        double t4;
        double t6;
        double trm1;
        double trm2;
        double trm3;
        double gce;
        double trm4;
        double gcn;

        dlon = ln - cm;
        while (dlon > PI) {
            dlon -= TWOPI;
        }
        while (dlon < -PI) {
            dlon += TWOPI;
        }

        m = meridian_arc(tm, lt);

        slt = Math.sin(lt);

        eslt = (1.0 - e2 * slt * slt);
        eta = a / Math.sqrt(eslt);
        rho = eta * (1.0 - e2) / eslt;
        psi = eta / rho;

        clt = Math.cos(lt);
        w = dlon;

        wc = clt * w;
        wc2 = wc * wc;

        t = slt / clt;
        t2 = t * t;
        t4 = t2 * t2;
        t6 = t2 * t4;

        trm1 = (psi - t2) / 6.0;

        trm2 = (((4.0 * (1.0 - 6.0 * t2) * psi
                + (1.0 + 8.0 * t2)) * psi
                - 2.0 * t2) * psi + t4) / 120.0;

        trm3 = (61 - 479.0 * t2 + 179.0 * t4 - t6) / 5040.0;

        gce = (sf * eta * dlon * clt) * (((trm3 * wc2 + trm2) * wc2 + trm1) * wc2 + 1.0);
        out.ce = gce / utom + fe;

        trm1 = 1.0 / 2.0;

        trm2 = ((4.0 * psi + 1) * psi - t2) / 24.0;

        trm3 = ((((8.0 * (11.0 - 24.0 * t2) * psi
                - 28.0 * (1.0 - 6.0 * t2)) * psi
                + (1.0 - 32.0 * t2)) * psi
                - 2.0 * t2) * psi
                + t4) / 720.0;

        trm4 = (1385.0 - 3111.0 * t2 + 543.0 * t4 - t6) / 40320.0;

        gcn = (eta * t) * ((((trm4 * wc2 + trm3) * wc2 + trm2) * wc2 + trm1) * wc2);
        out.cn = (gcn + m - om) * sf / utom + fn;

        return out;
    }

    /* Define a static implementation of tmprojection */
    /* Note: for some implementations it may be better to create this
    dynamically and develop modified versions of the transformation
    functions to take this as a parameter */
    tmprojection nztm_projection;
    boolean initiallized = false;

    tmprojection get_nztm_projection() {
        if (!initiallized) {
            nztm_projection = new tmprojection();
            define_tmprojection(nztm_projection, NZTM_A, NZTM_RF,
                    NZTM_CM / rad2deg, NZTM_SF, NZTM_OLAT / rad2deg, NZTM_FE, NZTM_FN,
                    1.0);
            initiallized = true;
        }
        return nztm_projection;
    }

    /* Functions implementation the TM projection specifically for the
    NZTM coordinate system
     */

    /*void nztm_geod( double n, double e, double *lt, double *ln )
    {
    tmprojection nztm = get_nztm_projection();
    tm_geod( nztm, e, n, ln, lt );
    }*/
    public TMpoint geod_nztm(double lt, double ln) {
        tmprojection nztm = get_nztm_projection();
        return geod_tm(nztm, ln, lt);
    }

    public TMpoint geod_nztm_degrees(double lt, double ln) {
        tmprojection nztm = get_nztm_projection();
        return geod_tm(nztm, ln / rad2deg, lt / rad2deg);
    }
    
    class complex {

        public double real, imag;

        public complex(double re, double im) {
            real = re;
            imag = im;
        }

        public complex() {
        }
    }
    /*
    Coefficients for converting NZMG coordinates to longitude and latitude
    From L+S technical note set 1973/32
     */
    static double a = 6378388.0;
    static double n0 = 6023150.0;
    static double e0 = 2510000.0;
    static double lt0 = -41.0;
    static double ln0 = 173.0;
    static double cfi[] = {0.6399175073,
        -0.1358797613,
        0.063294409,
        -0.02526853,
        0.0117879,
        -0.0055161,
        0.0026906,
        -0.001333,
        0.00067,
        -0.00034};
    static double cfl[] = {1.5627014243,
        0.5185406398,
        -0.03333098,
        -0.1052906,
        -0.0368594,
        0.007317,
        0.01220,
        0.00394,
        -0.0013};
    complex cfb1[] = {new complex(0.7557853228, 0.0),
        new complex(0.249204646, 0.003371507),
        new complex(-0.001541739, 0.041058560),
        new complex(-0.10162907, 0.01727609),
        new complex(-0.26623489, -0.36249218),
        new complex(-0.6870983, -1.1651967)};
    complex cfb2[] = {new complex(1.3231270439, 0.0),
        new complex(-0.577245789, -0.007809598),
        new complex(0.508307513, -0.112208952),
        new complex(-0.15094762, 0.18200602),
        new complex(1.01418179, 1.64497696),
        new complex(1.9660549, 2.5127645)};


    /*----------------------------------------------------------------*/
    /*                                                                */
    /*  Basic complex arithmetic routines                             */
    /*                                                                */
    /*----------------------------------------------------------------*/
    complex cadd(complex cr, complex c1, complex c2) {
        cr.real = c1.real + c2.real;
        cr.imag = c1.imag + c2.imag;
        return cr;
    }

    complex csub(complex cr, complex c1, complex c2) {
        cr.real = c1.real - c2.real;
        cr.imag = c1.imag - c2.imag;
        return cr;
    }

    complex cmult(complex cr, complex c1, complex c2) {
        complex temp = new complex();
        temp.real = c1.real * c2.real - c1.imag * c2.imag;
        temp.imag = c1.real * c2.imag + c1.imag * c2.real;
        cr.real = temp.real;
        cr.imag = temp.imag;
        return cr;
    }

    complex cdiv(complex cr, complex c1, complex c2) {
        complex temp = new complex();
        double cmod2;
        cmod2 = (c2.real * c2.real + c2.imag * c2.imag);
        temp.real = c2.real / cmod2;
        temp.imag = -c2.imag / cmod2;
        cmult(cr, c1, temp);
        return cr;
    }

    complex cscale(complex cr, complex c1, double sc) {
        cr.real = c1.real * sc;
        cr.imag = c1.imag * sc;
        return cr;
    }

    /*----------------------------------------------------------------*/
    /*                                                                */
    /*  Routines to do the conversions to and from NZMG               */
    /*                                                                */
    /*----------------------------------------------------------------*/

    /*void nzmg_geod( double n, double e, double *lt, double *ln ) {
    complex z0, z1, zn, zd, tmp1, tmp2;
    double sum,tmp;
    short i, it;
    
    z0.real = (n-n0)/a;     z0.imag = (e-e0)/a;
    z1.real = cfb2[5].real; z1.imag = cfb2[5].imag;
    for (i=5; i--; ) cadd(&z1, cmult(&z1, &z1, &z0), cfb2+i );
    cmult(&z1,&z1,&z0);
    
    for(it=2; it--; ) {
    cscale( &zn, cfb1+5, 5.0);
    cscale( &zd, cfb1+5, 6.0);
    for (i=4; i; i--) {
    cadd( &zn, cmult(&tmp1, &zn, &z1), cscale(&tmp2, cfb1+i, (double) i));
    cadd( &zd, cmult(&tmp1, &zd, &z1), cscale(&tmp2, cfb1+i, (double) (i+1)));
    }
    cadd( &zn, &z0, cmult( &zn, cmult( &zn, &zn, &z1), &z1));
    cadd( &zd, cfb1, cmult( &zd, &zd, &z1 ));
    cdiv( &z1, &zn, &zd );
    }
    
     *ln = ln0/rad2deg + z1.imag;
    
    tmp = z1.real;
    sum = cfl[8];
    for (i=8; i--;) sum = sum*tmp + cfl[i];
    sum *= tmp/3600.0e-5;
     *lt = (lt0+sum)/rad2deg;
    }*/
    TMpoint geod_nzmg(double lt, double ln) {
        double sum;
        int i;
        complex z0 = new complex(), z1 = new complex();

        lt = (lt * rad2deg - lt0) * 3600.0e-5;
        sum = cfi[9];
        for (i = 8; i >= 0; i--) {
            sum = sum * lt + cfi[i];
        }
        sum *= lt;

        z1.real = sum;
        z1.imag = ln - ln0 / rad2deg;
        z0.real = cfb1[5].real;
        z0.imag = cfb1[5].imag;
        for (i = 4; i >= 0; i--) { //for (i = 5; i--;) {
            cadd(z0, cmult(z0, z0, z1), cfb1[i]);
        }
        cmult(z0, z0, z1);

        TMpoint p = new TMpoint();
        p.cn = n0 + z0.real * a;
        p.ce = e0 + z0.imag * a;
        return p;
    }

    TMpoint geod_nzmg_degrees(double lt, double ln) {
        return geod_nzmg(lt / rad2deg, ln / rad2deg);
    }
}

/**
 * @author Tom and Hil
 */
public class NZGPS_MIDlet extends MIDlet implements CommandListener {

    private Display display;
    //private Command cmdExit, cmdOK;
    private StringItem si;
    private boolean midletPaused = false;
    MovementTracker M;
    //<editor-fold defaultstate="collapsed" desc=" Generated Fields ">//GEN-BEGIN:|fields|0|
    private Command exitCommand;
    private Command okCommand;
    private Command backCommand;
    private Form form;
    private StringItem stringItem;
    //</editor-fold>//GEN-END:|fields|0|

    private Command backCommand2;
    /**
     * The NZGPS_MIDlet constructor.
     */
    public NZGPS_MIDlet() {
        display = Display.getDisplay(this);
        form = new Form("NZ Topo GPS coordinates");
        exitCommand = new Command("Exit", Command.EXIT, 5);
        okCommand = new Command("Text size", Command.OK, 6);
        backCommand = new Command("About", Command.BACK, 7);
        backCommand2 = new Command("Back", Command.BACK, 8);
        //cmdOK = new Command("OK", Command.OK, 1);
        si = new StringItem("", cachedString);
        form.append(si);
        
        //form.addCommand(cmdOK);
        //form.addCommand(exitCommand);
        form.addCommand(okCommand);
        form.addCommand(backCommand);
        form.setCommandListener(this);

        try {
            M = new MovementTracker(this);
        } catch (Exception e) {
            si.setText("Error: " + e.getMessage());
        }
    }

    //<editor-fold defaultstate="collapsed" desc=" Generated Methods ">//GEN-BEGIN:|methods|0|
    //</editor-fold>//GEN-END:|methods|0|
    //<editor-fold defaultstate="collapsed" desc=" Generated Method: initialize ">//GEN-BEGIN:|0-initialize|0|0-preInitialize
    /**
     * Initializes the application.
     * It is called only once when the MIDlet is started. The method is called before the <code>startMIDlet</code> method.
     */
    private void initialize() {//GEN-END:|0-initialize|0|0-preInitialize
        // write pre-initialize user code here
//GEN-LINE:|0-initialize|1|0-postInitialize
        // write post-initialize user code here
        try {
            //checkLocation();
        } catch (Exception e) {
        }
    }//GEN-BEGIN:|0-initialize|2|
    //</editor-fold>//GEN-END:|0-initialize|2|

    //<editor-fold defaultstate="collapsed" desc=" Generated Method: startMIDlet ">//GEN-BEGIN:|3-startMIDlet|0|3-preAction
    /**
     * Performs an action assigned to the Mobile Device - MIDlet Started point.
     */
    public void startMIDlet() {//GEN-END:|3-startMIDlet|0|3-preAction
        // write pre-action user code here
        switchDisplayable(null, getForm());//GEN-LINE:|3-startMIDlet|1|3-postAction
        // write post-action user code here
    }//GEN-BEGIN:|3-startMIDlet|2|
    //</editor-fold>//GEN-END:|3-startMIDlet|2|

    //<editor-fold defaultstate="collapsed" desc=" Generated Method: resumeMIDlet ">//GEN-BEGIN:|4-resumeMIDlet|0|4-preAction
    /**
     * Performs an action assigned to the Mobile Device - MIDlet Resumed point.
     */
    public void resumeMIDlet() {//GEN-END:|4-resumeMIDlet|0|4-preAction
        // write pre-action user code here
//GEN-LINE:|4-resumeMIDlet|1|4-postAction
        // write post-action user code here
    }//GEN-BEGIN:|4-resumeMIDlet|2|
    //</editor-fold>//GEN-END:|4-resumeMIDlet|2|

    //<editor-fold defaultstate="collapsed" desc=" Generated Method: switchDisplayable ">//GEN-BEGIN:|5-switchDisplayable|0|5-preSwitch
    /**
     * Switches a current displayable in a display. The <code>display</code> instance is taken from <code>getDisplay</code> method. This method is used by all actions in the design for switching displayable.
     * @param alert the Alert which is temporarily set to the display; if <code>null</code>, then <code>nextDisplayable</code> is set immediately
     * @param nextDisplayable the Displayable to be set
     */
    public void switchDisplayable(Alert alert, Displayable nextDisplayable) {//GEN-END:|5-switchDisplayable|0|5-preSwitch
        // write pre-switch user code here
        Display display = getDisplay();//GEN-BEGIN:|5-switchDisplayable|1|5-postSwitch
        if (alert == null) {
            display.setCurrent(nextDisplayable);
        } else {
            display.setCurrent(alert, nextDisplayable);
        }//GEN-END:|5-switchDisplayable|1|5-postSwitch
        // write post-switch user code here
    }//GEN-BEGIN:|5-switchDisplayable|2|
    //</editor-fold>//GEN-END:|5-switchDisplayable|2|

    private int fontState = 0;
    private boolean displayAbout = false;
    private String cachedString = "Searching for GPS signal (allow up-to 3 minutes, and ensure the device has a clear view of the sky)...";
    
    //<editor-fold defaultstate="collapsed" desc=" Generated Method: commandAction for Displayables ">//GEN-BEGIN:|7-commandAction|0|7-preCommandAction
    /**
     * Called by a system to indicated that a command has been invoked on a particular displayable.
     * @param command the Command that was invoked
     * @param displayable the Displayable where the command was invoked
     */
    public void commandAction(Command command, Displayable displayable) {//GEN-END:|7-commandAction|0|7-preCommandAction
        // write pre-action user code here
        if (displayable == form) {//GEN-BEGIN:|7-commandAction|1|25-preAction
            if (command == backCommand) {//GEN-END:|7-commandAction|1|25-preAction
                // write pre-action user code here
                form.removeCommand(backCommand);
                //form.removeCommand(okCommand);
                form.addCommand(backCommand2);
                displayAbout = true;
                si.setText("NZ Topo GPS coordinates v1.0\n\nConvert coordinates from the phone's GPS into coordinates for NZ topo maps.\n\n"
                        + "To install on a different phone, enter http://hilandtom.com/gps.jar in the phone's browser.\n\n"
                        + "Select \"flight mode\" to prolong battery life when away from phone signals.\n\n"
                        + "A phone will generally take longer to find a signal, and is more likely to fail, than a dedicated GPS unit (these have better aerials and chips). Ensure it has a good view of the sky.\n\n"
                        + "Use the \"hang-up\" button to exit.\n\n"
                        + "More info at http://hilandtom.com/gps");
            }
            else if (command == backCommand2) {
                form.addCommand(backCommand);
                //form.addCommand(okCommand);
                form.removeCommand(backCommand2);
                si.setText(cachedString);
                displayAbout = false;                
//GEN-LINE:|7-commandAction|2|25-postAction
                // write post-action user code here
            } else if (command == exitCommand) {//GEN-LINE:|7-commandAction|3|19-preAction
                // write pre-action user code here
                exitMIDlet();//GEN-LINE:|7-commandAction|4|19-postAction
                // write post-action user code here
            } else if (command == okCommand) {//GEN-LINE:|7-commandAction|5|23-preAction
                // write pre-action user code here
//GEN-LINE:|7-commandAction|6|23-postAction
                // write post-action user code here
                fontState++; fontState %= 3;
                int aSizes[] = {Font.SIZE_SMALL,  Font.SIZE_MEDIUM, Font.SIZE_LARGE};
                si.setFont(Font.getFont(Font.FACE_SYSTEM, Font.STYLE_PLAIN, aSizes[fontState]));
            }//GEN-BEGIN:|7-commandAction|7|7-postCommandAction
        }//GEN-END:|7-commandAction|7|7-postCommandAction
        // write post-action user code here
    }//GEN-BEGIN:|7-commandAction|8|
    //</editor-fold>//GEN-END:|7-commandAction|8|

    //<editor-fold defaultstate="collapsed" desc=" Generated Getter: exitCommand ">//GEN-BEGIN:|18-getter|0|18-preInit
    /**
     * Returns an initiliazed instance of exitCommand component.
     * @return the initialized component instance
     */
    public Command getExitCommand() {
        if (exitCommand == null) {//GEN-END:|18-getter|0|18-preInit
            // write pre-init user code here
            exitCommand = new Command("Exit", Command.EXIT, 0);//GEN-LINE:|18-getter|1|18-postInit
            // write post-init user code here
        }//GEN-BEGIN:|18-getter|2|
        return exitCommand;
    }
    //</editor-fold>//GEN-END:|18-getter|2|

    //<editor-fold defaultstate="collapsed" desc=" Generated Getter: form ">//GEN-BEGIN:|14-getter|0|14-preInit
    /**
     * Returns an initiliazed instance of form component.
     * @return the initialized component instance
     */
    public Form getForm() {
        if (form == null) {//GEN-END:|14-getter|0|14-preInit
            // write pre-init user code here
            form = new Form("Welcome", new Item[] { getStringItem() });//GEN-BEGIN:|14-getter|1|14-postInit
            form.addCommand(getExitCommand());
            form.addCommand(getOkCommand());
            form.addCommand(getBackCommand());
            form.setCommandListener(this);//GEN-END:|14-getter|1|14-postInit
            // write post-init user code here
        }//GEN-BEGIN:|14-getter|2|
        return form;
    }
    //</editor-fold>//GEN-END:|14-getter|2|

    //<editor-fold defaultstate="collapsed" desc=" Generated Getter: stringItem ">//GEN-BEGIN:|16-getter|0|16-preInit
    /**
     * Returns an initiliazed instance of stringItem component.
     * @return the initialized component instance
     */
    public StringItem getStringItem() {
        if (stringItem == null) {//GEN-END:|16-getter|0|16-preInit
            // write pre-init user code here
            stringItem = new StringItem("Hello", "Hello, World!", Item.PLAIN);//GEN-LINE:|16-getter|1|16-postInit
            // write post-init user code here
        }//GEN-BEGIN:|16-getter|2|
        return stringItem;
    }
    //</editor-fold>//GEN-END:|16-getter|2|

    //<editor-fold defaultstate="collapsed" desc=" Generated Getter: okCommand ">//GEN-BEGIN:|22-getter|0|22-preInit
    /**
     * Returns an initiliazed instance of okCommand component.
     * @return the initialized component instance
     */
    public Command getOkCommand() {
        if (okCommand == null) {//GEN-END:|22-getter|0|22-preInit
            // write pre-init user code here
            okCommand = new Command("Text size", Command.OK, 0);//GEN-LINE:|22-getter|1|22-postInit
            // write post-init user code here
        }//GEN-BEGIN:|22-getter|2|
        return okCommand;
    }
    //</editor-fold>//GEN-END:|22-getter|2|

    //<editor-fold defaultstate="collapsed" desc=" Generated Getter: backCommand ">//GEN-BEGIN:|24-getter|0|24-preInit
    /**
     * Returns an initiliazed instance of backCommand component.
     * @return the initialized component instance
     */
    public Command getBackCommand() {
        if (backCommand == null) {//GEN-END:|24-getter|0|24-preInit
            // write pre-init user code here
            backCommand = new Command("About", Command.BACK, 0);//GEN-LINE:|24-getter|1|24-postInit
            // write post-init user code here
        }//GEN-BEGIN:|24-getter|2|
        return backCommand;
    }
    //</editor-fold>//GEN-END:|24-getter|2|

    /**
     * Returns a display instance.
     * @return the display instance.
     */
    public Display getDisplay() {
        return Display.getDisplay(this);
    }

    /**
     * Exits MIDlet.
     */
    public void exitMIDlet() {
        switchDisplayable(null, null);
        destroyApp(true);
        notifyDestroyed();
    }

    /**
     * Called when MIDlet is started.
     * Checks whether the MIDlet have been already started and initialize/starts or resumes the MIDlet.
     */
    public void startApp() {
        if (midletPaused) {
            resumeMIDlet();
        } else {
            initialize();
            startMIDlet();
        }
        midletPaused = false;
    }

    /**
     * Called when MIDlet is paused.
     */
    public void pauseApp() {
        midletPaused = true;
    }

    /**
     * Called to signal the MIDlet to terminate.
     * @param unconditional if true, then the MIDlet has to be unconditionally terminated and all resources has to be released.
     */
    public void destroyApp(boolean unconditional) {
    }

    static int round(double d) {
        Double d_rounded = new Double(Math.floor(d + 0.5));
        return d_rounded.intValue();
    }
    
    static String round(double d, int dp)
    {
        int scale = 1;
        for(int i=0;i<dp;i++)
            scale *= 10;
        
        int d_rounded = round(d*scale);
        String s = d_rounded/scale + "." + d_rounded % scale;
        return s;
    }
    
    static String pad(int i)
    {
        if(i<10) return "00" + i;
        if(i<100) return "0" + i;
        return "" + i;
    }

    static String sixFig(convertCoords.TMpoint p) {
        int N = (round(p.cn / 100.0)) % 1000;
        int E = (round(p.ce / 100.0)) % 1000;
        String s = "E" + pad(E) + "N" + pad(N) + "\n";
        return s;
    }

    static String DMS(double deg) {
        int sign = (deg>0) ? 1 : -1;
        deg = deg * sign;
        int D = (int)Math.floor(deg);
        double minutes = 60.0*(deg - D);
        int M = (int)Math.floor(minutes);
        double seconds = (minutes-M)*60.0;
        int S = round(seconds);
        String s = sign * D + "\u00B0" + M + "'" + S + "\"";
        return s;
    }
    
    static String aDirs[] = {"N","NNE","NE","ENE","E","ESE","SE","SSE","S","SSW","SW","WSW","W","WNW","NW","NNW"};
    static String getHeading(float bearing)
    {
        int sector = (int) Math.floor((bearing + 360.0/32.0) * (16.0/360.0));
        return aDirs[sector];
    }
    
    static String getHeadingStr(Location l)
    {
        //if(l.getSpeed() > 0.2)
            return round(l.getCourse()) + "\u00B0 (" + getHeading(l.getCourse());
        //else
            //return "Not enough movement";
    }
    static boolean inNZ(Coordinates l)
    {
        return (l.getLatitude() < -33) && (l.getLatitude() > -52) && (l.getLongitude() > 165) && (l.getLongitude() < 180);
    }

    public void checkLocation(Location l) throws Exception {
        String string;
        //Location l;
        //LocationProvider lp;
        
        // Set criteria for selecting a location provider:
        // accurate to 500 meters horizontally
        //Criteria cr = new Criteria();
        //cr.setHorizontalAccuracy(500);
        //cr.getHorizontalAccuracy();

        // Get an instance of the provider
        //lp = LocationProvider.getInstance(cr);

        // Request the location, setting a one-minute timeout
        //l = lp.getLocation(60);
        Date date = new Date();
        string = date.toString();

        Coordinates c = l.getQualifiedCoordinates();

        if (c != null) {
            // Use coordinate information
            double lat = c.getLatitude();
            double lon = c.getLongitude();
            string = string + "\nLat:" + DMS(lat) + " Long:" + DMS(lon) + " Alt:" + c.getAltitude() + "m Error (1 s.d. horizontal): " + l.getQualifiedCoordinates().getHorizontalAccuracy() + "m\n"; //l.getExtraInfo("text/plain") is null
            
            if(inNZ(c))
            {
                try {
                    convertCoords CC = new convertCoords();
                    //lat = -(41 + 1 / 6.0); to about 441,425
                    //lon = 172 + 2.0 / 6.0;
                    convertCoords.TMpoint nztmpoint = CC.geod_nztm_degrees(lat, lon);
                    string = string + "\nNZTM2000 (new): " + sixFig(nztmpoint) + "(E:" + round(nztmpoint.ce) + " N:" + round(nztmpoint.cn) + ")\n";
                } catch (Exception e) {
                    string = string + "\nCould not convert to NZTM (" + e.getMessage() + ")\n";
                }

                try {
                    convertCoords CC = new convertCoords();
                    //lat = -(43 + 32.0 / 60.0); //to about 349 395
                    //lon = 170 + 50.0 / 60.0;
                    convertCoords.TMpoint nzmgpoint = CC.geod_nzmg_degrees(lat, lon);
                    string = string + "\nNZMG1949 (old): " + sixFig(nzmgpoint) + "(E:" + round(nzmgpoint.ce) + " N:" + round(nzmgpoint.cn) + ")";
                } catch (Exception e) {
                    string = string + "\nCould not convert to NZMG (" + e.getMessage() + ")";
                }
            }
            else
                string = string + "Not in NZ\n";
            
            string = string + "\n\nSpeed: " + round(l.getSpeed()*3.6, 1) + "km/h, Heading: " + getHeadingStr(l) + ")\n";

        } else {
            string = string + "\n\nFailed to get satellite signal--trying again..." + "\n\nMake sure the device has a clear view of the sky.";
        }
        if(!displayAbout) 
            si.setText(string);
        else
            cachedString = string;
        
    }
}

class MovementTracker implements LocationListener {

    float movementDistance;
    LocationProvider provider;
    Location lastValidLocation;
    UpdateHandler handler;
    boolean done;
    NZGPS_MIDlet HM;

    public MovementTracker(NZGPS_MIDlet HM_in) throws LocationException {
        HM = HM_in;
        this.movementDistance = 5;
        done = false;
        handler = new UpdateHandler();
        new Thread(handler).start();
        
        Criteria criteria = new Criteria();
        criteria.setSpeedAndCourseRequired(true);
        criteria.setAltitudeRequired(true);
        criteria.setHorizontalAccuracy(500);
        provider = LocationProvider.getInstance(criteria);
        provider.setLocationListener(this, -1, -1, -1);
    }

    public void locationUpdated(LocationProvider provider, Location location) {
        handler.handleUpdate(location);
    }

    public void providerStateChanged(LocationProvider provider, int newState) {
    }

    class UpdateHandler implements Runnable {

        private Location updatedLocation = null;

        // The run method performs the actual processing of the location 
        // updates
        public void run() {
            Location locationToBeHandled = null;
            while (!done) {
                synchronized (this) {
                    if (updatedLocation == null) {
                        try {
                            wait();
                        } catch (Exception e) {
                            // Handle interruption
                        }
                    }
                    locationToBeHandled = updatedLocation;
                    updatedLocation = null;
                }

                // The benefit of the MessageListener is here.
                // This thread could via similar triggers be
                // handling other kind of events as well in
                // addition to just receiving the location updates.
                if (locationToBeHandled != null) {
                    processUpdate(locationToBeHandled);
                }
            }
        }

        public synchronized void handleUpdate(Location update) {
            updatedLocation = update;
            //notify();
            processUpdate(update);
        }

        private void processUpdate(Location update) {
            if (true || update.getQualifiedCoordinates().distance(
                    lastValidLocation.getQualifiedCoordinates())
                    > movementDistance) {
                // Alert user to movement...

                // Cache new position as we have moved a sufficient distance 
                // from last one
                lastValidLocation = update;
                try {
                    HM.checkLocation(lastValidLocation);
                } catch (Exception e) {
                }
            }
        }
    }
}